"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Commander = void 0;
// tslint:disable:no-console
const chalk = require("chalk");
const cli = require("commander");
const fs = require("fs-extra");
const glob = require("glob");
const path = require("path");
const commander_1 = require("./commander");
const modules_1 = require("./modules");
class Commander {
    ioc(iocContainer) {
        (0, commander_1.setIocContainer)(iocContainer);
        return this;
    }
    commandsFromDirectory(directoryPath) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                const pattern = path.join(directoryPath, '**/*.@(js|ts)');
                glob(pattern, (err, matches) => {
                    if (err) {
                        return reject(err);
                    }
                    const moduleFileNames = (0, modules_1.getUniqueModuleNames)(matches);
                    for (const moduleFileName of moduleFileNames) {
                        require(moduleFileName);
                    }
                    resolve(this);
                });
            });
        });
    }
    version(version) {
        this._version = version;
        return this;
    }
    versionFromPackage(directoryPath) {
        this._version = this.getPackageVersion(path.resolve(directoryPath, './package.json'));
        return this;
    }
    execute(argv) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            if (this._version) {
                cli.program.version(this._version);
            }
            const args = argv || process.argv;
            cli.program.addHelpText('after', '\n');
            if (args.length <= 2) {
                cli.program.outputHelp();
                return;
            }
            cli.program.on('command:*', () => {
                console.error();
                console.error('Invalid command: %s\nSee --help for a list of available commands.', cli.program.args.join(' '));
                console.error();
                process.exitCode = 1;
            });
            try {
                yield cli.program.parseAsync(args);
            }
            catch (err) {
                process.exitCode = 1;
                console.error();
                // istanbul ignore next
                const errorMessage = (_a = err.message) !== null && _a !== void 0 ? _a : err;
                console.error(chalk.red(errorMessage));
                console.error();
                throw err;
            }
        });
    }
    getPackageVersion(packageFileName) {
        const exists = fs.pathExistsSync(packageFileName);
        if (!exists) {
            throw new Error('Could not find package.json to load version from');
        }
        const packageObject = fs.readJSONSync(packageFileName);
        const version = packageObject && packageObject.version;
        return version;
    }
}
exports.Commander = Commander;
