"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCommandUsage = exports.getCommandDefinitionUsage = exports.registerCommand = exports.getIocContainer = exports.setIocContainer = void 0;
const cli = require("commander");
const coercion_1 = require("./coercion");
const metadata_1 = require("./metadata");
let iocContainer;
const commandDefinitions = [];
function setIocContainer(container) {
    iocContainer = container;
}
exports.setIocContainer = setIocContainer;
function getIocContainer() {
    return iocContainer;
}
exports.getIocContainer = getIocContainer;
function getOptionUsage(option) {
    let result = '';
    if (option.shortName) {
        result = `-${option.shortName}, `;
    }
    result += `--${option.name.toString()} `;
    if (option.valueName) {
        result += `<${option.valueName}>`;
    }
    return result.trim();
}
function getParams(command, args) {
    const values = (0, metadata_1.getCommandValues)(command.paramsClass.prototype);
    const options = (0, metadata_1.getCommandOptions)(command.paramsClass.prototype);
    const params = new command.paramsClass();
    let paramIndex = 0;
    for (const value of values) {
        const argValue = args[paramIndex++];
        const defaultValue = params[value.name];
        const paramValue = (0, coercion_1.coerceValue)(argValue, value.type, defaultValue);
        params[value.name] = paramValue;
    }
    const optionValues = args[paramIndex];
    for (const option of options) {
        const value = optionValues[option.name];
        if (value !== undefined) {
            params[option.name.toString()] = value;
        }
    }
    return params;
}
function instantiateCommand(command) {
    const constructor = command.type;
    const instance = iocContainer ? iocContainer.get(constructor) : new constructor();
    return instance;
}
function registerCommandOption(cliCommand, paramsClass, option) {
    var _a;
    const optionUsage = getOptionUsage(option);
    const coercedValue = ((value) => {
        if (option.valueName && option.type === Number) {
            return +value;
        }
        else {
            return value;
        }
    });
    const params = new paramsClass();
    const defaultValue = params[option.name];
    cliCommand.option(optionUsage, (_a = option.description) !== null && _a !== void 0 ? _a : '', coercedValue, defaultValue);
}
function registerCommand(commandDefinition) {
    commandDefinitions.push(commandDefinition);
    const usage = getCommandDefinitionUsage(commandDefinition);
    const cliCommand = cli.program.command(usage);
    if (commandDefinition.description) {
        cliCommand.description(commandDefinition.description);
    }
    const options = (0, metadata_1.getCommandOptions)(commandDefinition.paramsClass.prototype);
    for (const option of options) {
        registerCommandOption(cliCommand, commandDefinition.paramsClass, option);
    }
    cliCommand.action((...args) => __awaiter(this, void 0, void 0, function* () {
        const commandInstance = instantiateCommand(commandDefinition);
        const params = getParams(commandDefinition, args);
        yield commandInstance.execute(params);
    }));
}
exports.registerCommand = registerCommand;
function valueDefinitionToUsageString(value) {
    const name = `${value.name}${value.variadic ? '...' : ''}`;
    const usage = value.optional ? `[${name}]` : `<${name}>`;
    return usage;
}
function getCommandDefinitionUsage(commandDefinition) {
    const values = (0, metadata_1.getCommandValues)(commandDefinition.paramsClass.prototype);
    const valuesUsage = values
        .map((value, index) => ({
        value,
        index
    }))
        .sort((a, b) => a.value.variadic ? 1 : b.value.variadic ? -1 : a.index - b.index)
        .map((valueAndIndex) => valueDefinitionToUsageString(valueAndIndex.value))
        .join(' ');
    const usage = `${commandDefinition.name} ${valuesUsage}`.trim();
    return usage;
}
exports.getCommandDefinitionUsage = getCommandDefinitionUsage;
function getCommandUsage(commandClass) {
    const commandDefinition = commandDefinitions
        .find((item) => item.type === commandClass);
    if (!commandDefinition) {
        throw new Error('Class is not a command');
    }
    const usage = getCommandDefinitionUsage(commandDefinition);
    return usage;
}
exports.getCommandUsage = getCommandUsage;
