"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.addCommandOption = exports.getCommandOptions = exports.addCommandValue = exports.getCommandValues = exports.metadataKeys = void 0;
require("reflect-metadata");
exports.metadataKeys = Object.freeze({
    command: Symbol('command'),
    options: Symbol('options'),
    values: Symbol('values')
});
function getCommandValues(paramsClassPrototype) {
    return [...(Reflect.getOwnMetadata(exports.metadataKeys.values, paramsClassPrototype) || [])];
}
exports.getCommandValues = getCommandValues;
function addCommandValue(options) {
    const { name, optional, paramsClassPrototype, variadicType } = options;
    const type = variadicType || Reflect.getMetadata('design:type', paramsClassPrototype, name);
    const variadic = !!variadicType;
    const values = getCommandValues(paramsClassPrototype);
    if (variadic && values.some((value) => value.variadic)) {
        throw new Error('Command can have only one variadic value');
    }
    values.push({
        name,
        type,
        optional,
        variadic
    });
    Reflect.defineMetadata(exports.metadataKeys.values, values, options.paramsClassPrototype);
}
exports.addCommandValue = addCommandValue;
function getCommandOptions(paramsClassPrototype) {
    return [...(Reflect.getOwnMetadata(exports.metadataKeys.options, paramsClassPrototype) || [])];
}
exports.getCommandOptions = getCommandOptions;
function addCommandOption(paramsClassPrototype, name, options) {
    const type = Reflect.getMetadata('design:type', paramsClassPrototype, name);
    const allOptions = getCommandOptions(paramsClassPrototype);
    allOptions.push(Object.assign({ name,
        type }, options));
    Reflect.defineMetadata(exports.metadataKeys.options, allOptions, paramsClassPrototype);
}
exports.addCommandOption = addCommandOption;
